;--------------------------------------------------------------------------
;    driver_FDC.s - generic FDC driver for WD279x (Z80)
;
;    Copyright (c) 2012 by Michal Hucik <http://www.ordoz.com>
;
;    Thies program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    driver_FDC is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with driver_FDC; if not, write to the Free Software
;    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
;--------------------------------------------------------------------------

 
;
; 0.1 - 2012-01-04:
;
; 	- pocatecni driver pro FDC WD279[1/3] bez upravy pro HD
;


	.module	driver_FDC


	.area	_CODE


;
; Pri MULTITASK_OS = 1 se budou volat DI a EI ihned po vstupu do driveru
;
MULTITASK_OS = 1

;
; Zpusob jak pozadame system o DI
;
;.macro SYSTEM_DI_REQUEST
;	di
;.endm

;
; Zpusob jak pozadame system o EI
;
;.macro SYSTEM_EI_REQUEST
;	ei
;.endm


;
; Zpusob jak pozadame system o DI-PIOZ80
;
;.macro SYSTEM_PIO_DI_REQUEST
;	ld a,#0x03
;	out (0xfc),a
;.endm

;
; Zpusob jak pozadame system o EI-PIOZ80
;
;.macro SYSTEM_PIO_EI_REQUEST
;	ld a,#0x83
;	out (0xfc),a
;.endm




; porty WD279x 
__FDC_CMD	= 0xd8	; out 
__FDC_STS	= 0xd8	; in 
__FDC_TRACK	= 0xd9	; in, out 
__FDC_SECTOR	= 0xda	; in, out 
__FDC_DATA	= 0xdb	; in, out 
__FDC_DRIVE	= 0xdc	; out 
__FDC_SIDE	= 0xdd	; out, 0. bit 
__FDC_DENSITY	= 0xde	; out, 0. bit 
__FDC_EINT	= 0xdf	; out, 0. bit 



; definice prikazu typ I.
__FDC_CMD_STEP_IN	= 0xa0
__FDC_CMD_STEP_OUT	= 0x80

__FDC_TYPI_HEAD_SPEED6	= 0x03
__FDC_TYPI_HEAD_SPEED12	= 0x02
__FDC_TYPI_HEAD_SPEED20	= 0x01
__FDC_TYPI_HEAD_SPEED30	= 0x00

__FDC_TYPI_NOVERIFY_TRK	= ( 1 << 2 )



; definice prikazu typ II.
__FDC_CMD_READ_SECTOR	= 0x60
__FDC_CMD_WRITE_SECTOR	= 0x40

__FDC_TYPII_MULTI_SEC	= ( 0 << 4 )
__FDC_TYPII_SINGLE_SEC	= ( 1 << 4 )

__FDC_TYPII_DLY_15MS	= ( 0 << 2 )	; lze pouzit i pro typ III.
__FDC_TYPII_NO_DLY_15MS	= ( 1 << 2 )	; lze pouzit i pro typ III.

__FDC_TYPII_VRF_SIDE	= ( 0 << 1 )
__FDC_TYPII_NO_VRF_SIDE	= ( 1 << 1 )

__FDC_TYPII_DATA_SIG_FB	= ( 1 << 0 )	; 0FBh (DAM)
__FDC_TYPII_DATA_SIG_F8	= ( 0 << 0 )	; 0F8h (vypustena DAM)


; definice prikazu typ III.
__FDC_CMD_READ_ADDR	= 0x3b


; definice prikazu typ IV.
__FDC_CMD_BREAK		= 0x2f


; ovladani mechaniky
__FDC_MOTOR_ON		= ( 1 << 7 )
__FDC_DRIVE_SELECT	= ( 1 << 2 )


;
;
; Konfiguracni parametry
;
;



;
;
; Timing diskovych operaci
; 
__FDC_Timing::

__FDC_Timing_driveOnTime:
	.dw	500
__FDC_Timing_trackTime:
	.db	6
__FDC_Timing_stepTime:
	.db	6
__FDC_Timing_motorPredef:
	.db	128


;
;
; Tabulka parametru jednotlivych disku
;
; 0. a 1. bit - velikost sektoru
;		0 - 128 B
;		1 - 256 B
;		2 - 512 B
;		3 - 1024 B
;
; 2. bit - pocet stran media
;		0 - jednostranne
;		1 - oboustranne
;
; 3. bit - invertovana data
;		0 - normalni data
;		1 - invertovana data
;
;__FDC_Drive:
;	.db	0x0e	; e = oboustranne medium, sector_size = 512, invertovana data
;	.db	0x0e
;	.db	0x0e
;	.db	0x0e



;
;
; Provozni parametry
;



;
; Nenulova hodnota znaci, ze motor prave bezi
;
__FDC_Motor::
	.db	0x00



;
; Prave vybrany disk - musi byt umisten bezprostredne pred __FDC_Requested_drive !!!
;
__FDC_last_drive:
;__FDC_Selected_drive:
	.db	0x00
;
; 
; Pozadovane parametry READ/WRITE operace
;
;__FDC_Requested:
;
;__FDC_Requested_drive:
;	.db	0x00
;__FDC_Requested_track:
;	.db	0x00
;__FDC_Requested_sector:
;	.db	0x00
;__FDC_Requested_dma:
;	.dw	0x0000




;
; Tady si drzime informaci o posledni nastavene stope
;
__FDC_trackTable:
	.db	0x00
	.db	0x00
	.db	0x00
	.db	0x00





;
;
; Sluzba - nasilne vypnouti motoru
;
;
_FDC_ForceMotorOff::
	xor a
	ld hl,#__FDC_Motor
	di			; SYSTEM_DI_REQUEST
	ld (hl),a
	ei			; SYSTEM_EI_REQUEST
	out (__FDC_DRIVE),a
	ret


;
;
; Sluzba -  IM2 odpocet pro prechod mechaniky do stand-by
;
; Tato rutina musi byt volana pouze uvnitr preruseni!
;
_FDC_IM2_service::
	xor a
	ld hl,#__FDC_Motor
	cp (hl)
	ret z			; motor nebezi
	dec (hl)
	ret nz			; jeste nenastal cas ho vypnout
	out (__FDC_DRIVE),a	; motor off
	ret




;
; Podle (__FDC_Requested_track) v zavislosti na 2. bitu parametru media
; urcime SIDE.
;
; Vystup:
;		Carry = 1 pokud je potreba nastavit SIDE
;		A     = ( SIDE << 3 ) pro verifikaci strany u prikazu R/W SECTOR
;
__FDC_DI_createSideFromTrack::

.if MULTITASK_OS
.else
	di				;SYSTEM_DI_REQUEST		; pozadame system o DI
.endif

__FDC_createSideFromTrack::

	xor a				; Carry = 0, overeni strany = 0
	bit 2,1 (ix)			; __FDC_Drive (params)
	ret z				; RET, pokud se jedna o jednostranne medium

	ld a, 2 (ix)			; __FDC_Requested_track
	rla
	rla
	rla
	and #0x08			; v A mame na 3.bitu pozadavek na overeni strany

	scf				; Carry = 1
	ret




;
; Pro zvoleny disk pripravi do HL definici velikosti sektoru. 
; Do C pripravi instrukci pro pripadne invertovani dat (cpl, nop)
;
__FDC_PrepareSectorSizeAndCpl::

	ld c,#0x00
	bit 3, 1 (ix)	; __FDC_Drive (params) - invertovane medium?
	jr z, 1$	; ne, data nebudou invertovana
	ld c,#0x2f	; cpl

	1$:


	; nastaveni velikosti sektoru podle 0. a 1. bitu
	ld a,1 (ix)	; __FDC_Drive (params)
	ld hl,#0x8001
	and #0x03
	ret z		; 00 = 0x8001 :  128 B
	ld h,#0x00
	cp #0x01
	ret z		; 01 = 0x0001 :  256 B
	inc l
	and #0x01
	ret z		; 10 = 0x0002 :  512 B
	sla a		; 10 = 0x0004 : 1024 B
	ret


 
;
;
; Sluzba - Precteni pozadovaneho sektoru
;
; Vstup:
;
;    FDC_DRIVE_ID,                       /* id zvolene mechaniky 0 - 3 */
;    FDC_DRIVE_PARAM,                    /* parametry media */
;    unsigned char log_track,            /* logicka stopa */ 
;    unsigned char phy_sector,           /* fyzicky sektor */
;    void* dma                           /* adresa dat */
;
;
; Vystup:
;		 0: OK
;		-1: ERROR
;
_FDC_ReadSector::
	push ix
	ld ix,#0x0004			; nastavime index 0 na 1 parametr funkce
	add ix, sp

.if MULTITASK_OS
	di					; SYSTEM_DI_REQUEST
.endif

	call __FDC_ReadSector_routine

.if MULTITASK_OS
	ei					; SYSTEM_EI_REQUEST
.endif

	pop ix
	ret


;
; Spolecna rutina pro cteni sektoru
;
__FDC_ReadSector_routine::

	call __FDC_PrepareSectorSizeAndCpl

	ld (#__FDC_ReadData_sector_size), hl	; nastavime velikost sektoru

	ld a,c
	ld (#__FDC_ReadData_cpl), a		; upravime DRQ rutinu

	
	call __FDC_driveON
	ld b,#0x05				; max pocet pokusu

__FDC_Read_nextAttempt:	
	push bc

	call __FDC_setSectorSideTrack		; nastav stopu, sector a stranu

	jr nc, __FDC_serviceRead_error		; Nastaveni se nezdarilo?

	call __FDC_DI_createSideFromTrack	; Zakazeme preruseni a do A pripravime overeni strany

	xor #__FDC_CMD_READ_SECTOR|__FDC_TYPII_SINGLE_SEC|__FDC_TYPII_NO_DLY_15MS|__FDC_TYPII_VRF_SIDE|__FDC_TYPII_DATA_SIG_FB|0x10

	out (__FDC_CMD),a

	call __FDC_waitForNotBusy
	call __FDC_ReadData			; Cteme data, pokud je vse OK, tak Carry = 1


.if MULTITASK_OS
.else
	ei					; SYSTEM_EI_REQUEST
.endif

	jr nc, __FDC_serviceRead_error		; Cteni bylo ukonceno s chybou?
	call __FDC_waitForCmdDone

	; zaverecne overeni statusu
	in a,(__FDC_STS)
	cpl
	and #0xbf				; maska podstatnych status bitu po READ_SECTOR
	scf					; pokud je to OK, tak Carry = 1
	jr z, __FDC_serviceReadWrite_done	; Status bez chyby?


__FDC_serviceRead_error:
	call __FDC_VerifyTrackByAddr		; overime format diskety a stopu podle ADDR

	pop bc
	djnz __FDC_Read_nextAttempt		; zkusime to cteni zopakovat 5x

	; cp/m tady ma volani RETRY/ABORT/IGNORE
	; v pripade, ze chceme proces zopakovat,
	; tak hupsneme zpet na _FDC_serviceRead

	ld l,#0xff				; return ( -1 )
	ret

__FDC_serviceReadWrite_done:
	pop bc
	ld l,a					; return ( A )
	ret




;
;
; Sluzba - Zapis pozadovaneho sektoru
;
; Vstup:
;
;    FDC_DRIVE_ID,                       /* id zvolene mechaniky 0 - 3 */
;    FDC_DRIVE_PARAM,                    /* parametry media */
;    unsigned char log_track,            /* logicka stopa */ 
;    unsigned char phy_sector,           /* fyzicky sektor */
;    void* dma                           /* adresa dat */
;
;
; Vystup:
;		 0: OK
;		-1: ERROR
;		-2: WR_PROTECT
;
_FDC_WriteSector::
	push ix
	ld ix,#0x0004			; nastavime index 0 na 1 parametr funkce
	add ix, sp

.if MULTITASK_OS
	di					; SYSTEM_DI_REQUEST
.endif

	call __FDC_WriteSector_routine

.if MULTITASK_OS
	ei					; SYSTEM_EI_REQUEST
.endif

	pop ix
	ret

 
__FDC_WriteSector_routine::

	call __FDC_PrepareSectorSizeAndCpl

	; u zapisu potrebujeme sector size prevest na skutecnou hodnotu
	ld a,l
	ld b,h
	bit 7,b
	jr z,1$

	xor a					; u 128 B nulujeme horni bajt 

	1$:
	ld h,a
	ld l,b

	ld (#__FDC_WriteData_sector_size), hl	; nastavime velikost sektoru

	ld a,c
	ld (#__FDC_WriteData_cpl), a		; upravime DRQ rutinu

	
	call __FDC_driveON
	ld b,#0x05				; max pocet pokusu

__FDC_Write_nextAttempt:	
	push bc

	call __FDC_setSectorSideTrack		; nastav stopu, sector a stranu

	jr nc, __FDC_serviceWrite_error		; Nastaveni se nezdarilo?

	
	; udelame test na WR_PRT
	call __FDC_SendBreakWhileIsBusy
	in a, (__FDC_STS)
	bit 6,a
	ld a,#0xfe
	jr z,__FDC_serviceReadWrite_done	; WR_PRT? return ( -2 )
	call __FDC_delay90us

	call __FDC_DI_createSideFromTrack	; Zakazeme preruseni a do A pripravime overeni strany

	xor #__FDC_CMD_WRITE_SECTOR|__FDC_TYPII_SINGLE_SEC|__FDC_TYPII_NO_DLY_15MS|__FDC_TYPII_VRF_SIDE|__FDC_TYPII_DATA_SIG_FB|0x10

	out (__FDC_CMD),a

	call __FDC_waitForNotBusy
	call __FDC_WriteData			; Dokud je BUSY, tak se pokousime zapisovat data


.if MULTITASK_OS
.else
	ei					; SYSTEM_EI_REQUEST
.endif


	ld a,#0x03
	call __FDC_delay1MSxA	

	; prenesli jsme ocekavany pocet bajtu?
	.db 0x01				; LD BC,nm
	__FDC_WriteData_sector_size:
	.dw 0x0000

	ld l,4 (ix)				; __FDC_Requested_dma
	ld h,5 (ix)				; do HL

	add hl,bc
	dec de
	scf
	ccf
	sbc hl,de
	jr nz, __FDC_serviceWrite_error		; pocet zapsanych bajtu nesouhlasi s velikosti sektoru!

	; zaverecne overeni statusu
	in a,(__FDC_STS)
	inc a
	scf					; pokud je vse OK, tak vratime Carry = 1
	jr z, __FDC_serviceReadWrite_done	; Status bez chyby? return ( 0 )


__FDC_serviceWrite_error:
	call __FDC_VerifyTrackByAddr		; overime format diskety a stopu podle ADDR

	pop bc
	djnz __FDC_Write_nextAttempt		; zkusime ten zapis zopakovat 5x

	; cp/m tady ma volani RETRY/ABORT/IGNORE
	; v pripade, ze chceme proces zopakovat,
	; tak hupsneme zpet na _FDC_serviceWrite

	ld l,#0xff				; return ( -1 )
	ret



;
; Dokud je BUSY, tak cekame na DRQ - kdyz prijde, tak zapiseme pripraveny bajt.
;
; Pokud zmizi BUSY, tak se vratime s Carry = 1 a rotovanym statusem v A
;
__FDC_WriteData::
	ld c,#__FDC_DATA

	ld e,4 (ix)		; __FDC_Requested_dma
	ld d,5 (ix)		; do DE

	ld hl,#__FDC_WriteData_prepare_byte

__FDC_WriteData_prepare_byte:
	ld a,(de)		; precti bajt z DMA
__FDC_WriteData_cpl:
	.db 0x00		; cpl, nebo nop
	inc de
	ld b,a			; do B pripraveny bajt

	1$:
	in a,(__FDC_STS)
	rra
	ret c			; uz neni BUSY, Carry = 1
	rra
	jr c,1$			; cekame na DRQ

	out (c),b		; zapis bajt
	jp (hl)			; jp (hl) ma jen 1 bajt a spotrebuje 4T


;
; Pokud prijde DRQ, tak precteme pozadovany pocet bajtu.
;
; Vraci:
;		Carry = 1: OK
;		Carry = 0: ERROR - timeout
;
__FDC_ReadData::
	.db 01			; LD BC,nm
__FDC_ReadData_sector_size:
	.dw 0x0002		; sector size
	ld de,#0x0000		; error timer

	ld l,4 (ix)		; __FDC_Requested_dma
	ld h,5 (ix)		; do HL

	; cekame na data
	__FDC_ReadData_dec_timer:
	dec de
	ld a,d
	or e
	ret z			; timeout, Carry = 0

	__FDC_ReadData_read_sts:
	in a,(__FDC_STS)
	and #0x02
	jr nz,__FDC_ReadData_dec_timer		; cekame na DRQ

	; DRQ rutina
	in a,(__FDC_DATA)

__FDC_ReadData_cpl:
	.db 0x00		; nop, nebo cpl

	ld (hl),a

	inc hl
	djnz __FDC_ReadData_read_sts
	dec c
	jp nz,__FDC_ReadData_read_sts

	scf			; OK, Carry = 1
	ret



;	
;
;
; Zapnuti pozadovaneho disku.
;
; Vstup:
;		IX
;		drive_id ulozeno v (__FDC_Requested_drive)
;
; Vystup:
;		-
;
__FDC_driveON::
	ld bc,#__FDC_last_drive
	ld a,(bc)

	ld de,#__FDC_trackTable
	ld h,#0x00
	ld l,a
	add hl,de

	in a,(__FDC_TRACK)
	ld (hl),a		; ulozit aktualni stopu disku, ktery byl naposledy vybran

	ld a,0 (ix)		; __FDC_Requested_drive

	ld (bc),a		; novy disk ulozit do __FDC_last_drive

	ld h,#0x00
	ld l,a
	add hl,de		; __FDC_trackTable nove vybraneho disku

	ld a,(hl)		; do A nacist predchozi stopu nove zvolene mechaniky

	out (__FDC_TRACK),a	; a vlozit ji do FDC
	
	ld a,(bc)		; novy drive_id do A

;
; Samotny start motoru, bez inicializace stopy
;
; drive_id je v A
;
;__FDC_init_driveON::
	or #__FDC_MOTOR_ON|__FDC_DRIVE_SELECT

.if MULTITASK_OS
.else
	di				; SYSTEM_DI_REQUEST		; pozadame system o DI
.endif

	out (__FDC_DRIVE),a


	ld a, (#__FDC_Timing_motorPredef)
	ld b,a
	ld hl,#__FDC_Motor
	ld a,(hl)
	ld (hl),b


.if MULTITASK_OS
.else
	ei				; SYSTEM_EI_REQUEST		; pozadame system o EI
.endif


	and a
	ret nz				; motor uz bezel, nemusime dal na nic cekat 

	ld hl,(#__FDC_Timing_driveOnTime)

	; pockame pozadovany pocet ms, nez se nam ustali otacky motoru
	1$:
	djnz 1$
	dec hl
	ld a,h
	or l
	jr nz,1$
	ret




;
; Podle (__FDC_Requested) a podle parametru aktivniho disku (media) nastavime sector, track a
; side.
;
; Vystup:
;		Carry = 1: OK
;		Carry = 0: ERROR
;
__FDC_setSectorSideTrack::
	ld a,3 (ix)			; __FDC_Requested_sector
	cpl
	out (__FDC_SECTOR),a

	call __FDC_createSideFromTrack	; v Carry obdrzime zda je potreba nastavit SIDE

	ld a,2 (ix)			; __FDC_Requested_track
	jr c,1$				; je potreba nastavit stranu?
	add a,a				; inverujeme 0. bit

	1$:
	out (__FDC_SIDE),a

	srl a
	ld h,a				; pozadovana stopa do H

	in a,(__FDC_TRACK)
	cpl
	sub h
	ccf				; Neguj Carry.
					; Pokud je v FDC pozadovana stopa,
					; tak Carry = 1.
					; V pripade, ze jsou odlisne, tak
					; Carry urcuje kterym smerem bude
					; potreba krokovat.

	ret z				; v FDC je jiz nastavena pozadovana stopa

	; Pokud je v FDC nastavena jina stopa, tak budeme 
	; krokovat pozadovanym smerem.

	ld l,a				; v L je rozdil mezi pozadovanou a
					; aktualni stopou

	jr nc,step_in$

	step_out$:

	call __FDC_StepOut
	dec l
	jr nz,step_out$
	jr step_done$

	step_in$:

	call __FDC_StepIn
	inc l
	jr nz,step_in$

	; krokovani dokonceno
	step_done$:
	in a,(__FDC_TRACK)		; overime zda uz je v FDC nastavena
	xor h				; pozadovana stopa
	add a,#1			; Pokud ano, Carry = 1.
					; Pri chybe Carry = 0.

;
; Pockame 1 ms * (__FDC_Timing_trackTime)
;
__FDC_delayByTrackSettingTime::
	ld a,(#__FDC_Timing_trackTime)
;
; Pockame 1 ms * A
;
__FDC_delay1MSxA::
	push bc
	ld b,#0x00
	1$:
	djnz 1$
	dec a
	jr nz,1$
	pop bc
	ret



;
; Overeni stopy
;
__FDC_VerifyTrackByAddr::
	ld a,#0xff
	; nemelo by se tohle delat v DI ???
	ld (#__FDC_Motor),a		; citac motoru na maximalni hodnotu

	ld h,#0x74			; timeout timer

	1$:
	in a,(__FDC_STS)		; az nebude BUSY
	rra
	jr c,2$				; tak proved overeni stopy

	dec hl
	ld a,h
	or l
	jr nz,1$

	jr __FDC_SendBreakWhileIsBusy	; timeout, nedockali jsme se NOT
					; BUSY, takze jdeme posilat BREAK, dokud to neozije


	; overeni stopy pomoci READ_ADDR
	2$:
	in a,(__FDC_TRACK)
	rra
	call c,__FDC_StepIn
	call nc,__FDC_StepOut

	call __FDC_delayByTrackSettingTime

.if MULTITASK_OS
.else
	di				; SYSTEM_DI_REQUEST
.endif

	ld a,#__FDC_CMD_READ_ADDR | __FDC_TYPII_NO_DLY_15MS
	out (__FDC_CMD),a

	call __FDC_ReadAddrData

.if MULTITASK_OS
.else
	ei				; SYSTEM_EI_REQUEST
.endif

	jr nc, __FDC_StepToTRK00	; READ_ADDR skoncil s chybou,
					; takze se pokusime udelat restore
	call __FDC_waitForCmdDone

	; zaverecna kontrola statusu
	in a,(__FDC_STS)
	cpl
	and #0xbf
	jr nz, __FDC_StepToTRK00	; spatny status? krokuj do TRK00
	
	ld a,c
	out (__FDC_TRACK),a		; nastavime stopu podle ADDR

	ret


;
; Precteni dat z ADDR
;
; Vraci:
;		Carry = 1: OK
;		Carry = 0: ERROR - timeout
;		reg. C   : prectena stopa
;
__FDC_ReadAddrData::
	ld l,#2				; stst maska
	ld d,#0x0f			; timeout timer

	1$:
	dec de
	ld a,d
	or e
	ret z				; timeout, Carry = 0

	in a,(__FDC_STS)
	and l
	jr nz,1$			; cekame na DRQ

	in a,(__FDC_DATA)
	ld c,a				; precteny track do C

	ld b,#5				; ted precteme a zahodime zbyvajicich 5 bajtu
	
	2$:
	in a,(__FDC_STS)
	and l
	jr nz, 2$
	
	in a,(__FDC_DATA)
	djnz 2$

	scf				; Carry = 1
	ret



;
; Budeme krokovat dolu, dokud nedojdeme na nultou stopu - Restore TRK00
;
;
__FDC_StepToTRK00::
	ld l,#0x50			; pocitame s tim, ze neni vic jak 80 stop

	1$:
	call __FDC_StepOut
	dec l
	jr z, __FDC_delayByTrackSettingTime
	in a,(__FDC_STS)
	bit 2,a
	jr nz, 1$			; opakuj dokud neni status TRK00

	ld a,#0xff
	out (__FDC_TRACK),a		; track = 0

	jr __FDC_delayByTrackSettingTime



;
; StepIN - krok o stopu nahoru
;
__FDC_StepIn::
	ld a,#__FDC_CMD_STEP_IN | __FDC_TYPI_HEAD_SPEED6 | __FDC_TYPI_NOVERIFY_TRK
	jr __FDC_StepInOut
;
; StepOUT - krok o stopu dolu
;
__FDC_StepOut::
	ld a,#__FDC_CMD_STEP_OUT | __FDC_TYPI_HEAD_SPEED6 | __FDC_TYPI_NOVERIFY_TRK

__FDC_StepInOut:
	out (__FDC_CMD),a

	ld a,(#__FDC_Timing_stepTime)
	call __FDC_delay1MSxA



;
; Dokud bude BUSY, tak budeme opakovane posilat __FDC_CMD_BREAK
;
__FDC_SendBreakWhileIsBusy::
	ld a,#__FDC_CMD_BREAK
	out (__FDC_CMD),a


;
; Cekame dokud je BUSY
;
__FDC_waitForCmdDone::
	call __FDC_delay90us

	1$:
	in a,(__FDC_STS)
	rra
	jr c,__FDC_delay90us		; pokud uz neni BUSY, tak wait a potom RET
	djnz 1$
	jr __FDC_SendBreakWhileIsBusy	; pokud je stale BUSY, tak posilej BREAK az do soudneho dne



;
; Pockame, dokud nezmizi priznak BUSY, nebo dokud B > 0
;
__FDC_waitForNotBusy::
	call __FDC_delay90us
	1$:
	in a,(__FDC_STS)
	and #0x81
	xor #0x80
	ret z		; not BUSY
	djnz 1$
	ret		; timeout


;
; Pockame cca 90 us
;
__FDC_delay90us::
	ld b,#0x18
	1$:
	djnz 1$
	ret




